Package: tigre
Version: 1.66.0
Date: 2021-08-04
Title: Transcription factor Inference through Gaussian process
        Reconstruction of Expression
Author: Antti Honkela, Pei Gao, Jonatan Ropponen, Miika-Petteri
        Matikainen, Magnus Rattray, Neil D. Lawrence
Maintainer: Antti Honkela <antti.honkela@helsinki.fi>
Depends: R (>= 2.11.0), BiocGenerics, Biobase
Imports: methods, AnnotationDbi, gplots, graphics, grDevices, stats,
        utils, annotate, DBI, RSQLite
Suggests: drosgenome1.db, puma, lumi, BiocStyle, BiocManager
Description: The tigre package implements our methodology of Gaussian
        process differential equation models for analysis of gene
        expression time series from single input motif networks. The
        package can be used for inferring unobserved transcription
        factor (TF) protein concentrations from expression measurements
        of known target genes, or for ranking candidate targets of a
        TF.
License: AGPL-3
URL: https://github.com/ahonkela/tigre
BugReports: https://github.com/ahonkela/tigre/issues
biocViews: Microarray, TimeCourse, GeneExpression, Transcription,
        GeneRegulation, NetworkInference, Bayesian
Config/pak/sysreqs: libpng-dev libxml2-dev libssl-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:34:33 UTC
RemoteUrl: https://github.com/bioc/tigre
RemoteRef: RELEASE_3_23
RemoteSha: 3987af6e456b1bf43e1269a86c532327a6af0aa2
NeedsCompilation: yes
Packaged: 2026-04-30 09:51:40 UTC; root
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-30 10:01:15 UTC; unix
