useDynLib("tigre", "_ClnDiffErfs")

importFrom(BiocGenerics, updateObject, getObjectSlots)

importClassesFrom(Biobase, AnnotatedDataFrame, AssayData, eSet, ExpressionSet,
                  MIAME, Versioned, VersionedBiobase, Versions)

importClassesFrom(methods, character, list, matrix, numeric)

importMethodsFrom(AnnotationDbi, as.list, colnames, exists, get, ncol,
                  revmap)

importMethodsFrom(Biobase, annotation, experimentData, exprs,
                 featureData, featureNames, phenoData)

importMethodsFrom(DBI, dbCommit, dbConnect, dbDisconnect, dbDriver,
                  dbExecute, dbGetQuery, dbListTables)

importMethodsFrom(methods, initialize, show)

importMethodsFrom(RSQLite, dbBeginTransaction)

importFrom(annotate, getAnnMap)

importFrom(Biobase, assayDataElement, assayDataElementNames,
          assayDataElementReplace, assayDataNew, contents)

importFrom(gplots, plotCI)

importFrom(graphics, layout, lines, par, plot, title, polygon)

importFrom(grDevices, dev.off, png)

importFrom(methods, callNextMethod, new, slotNames)

importFrom(RSQLite, SQLite)

importFrom(stats, optim, optimize, qgamma, qnorm, rnorm, sd)

importFrom(utils, sessionInfo, write.table, tail)

##export everything that does not start with a . or an _
exportPattern("^[^\\._]")
