\name{drosophila_mmgmos_fragment}
\docType{data}
\alias{drosophila_mmgmos_fragment}
\title{Fragment of 12 time point Drosophila embryonic development
  microarray gene expression time series}
\description{
  Four genes from the 12 time point Drosophila embryonic development
  Affymetrix microarray gene expression data set by Tomancak et
  al. (2002).

  The data has been processed using \code{mmgmos} from the
  \code{puma} package.
}
\usage{data(drosophila_mmgmos_fragment)}
\format{A \code{puma} package \code{exprReslt} object with
  3 repeats of the 12 time points for 4 probes.}
\source{\url{ftp://ftp.fruitfly.org/pub/embryo_tc_array_data/}}
\references{
  Tomancak, P et al.
  Systematic determination of patterns of gene expression during
  Drosophila embryogenesis.
  \emph{Genome Biol} 3:RESEARCH0088, 2002.
}
\keyword{datasets}
